#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/drivers/uart.h>
#include <stdio.h>
#include <string.h>


//add from RPI
#include <stdlib.h>
#include <stdbool.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>



#define EPC_LENGTH 25
#define MAX_EPC 115
#define DELAY_MS 150

/// just added - start
#define UART_RX_BUF_SIZE 128                 // Buffer size for received data
#define UART_RX_TIMEOUT 50                   // Timeout in milliseconds
#define RECEIVE_TIMEOUT 100

static uint8_t rx_buf[UART_RX_BUF_SIZE];
//static uint8_t rx_buf2[UART_RX_BUF_SIZE];
static volatile bool data_received = false;
// just added - end

typedef struct {
    unsigned char EPC[12];      // 12-byte EPC
    unsigned short initialRSSI; // Initial RSSI
    unsigned short timestamp;   // Timestamp in milliseconds
    unsigned char count;        // Count of detections
    unsigned short minRSSI;     // Minimum RSSI
    unsigned short maxRSSI;     // Maximum RSSI
} EPCRecord;

EPCRecord epcArray[MAX_EPC];  // Array to store EPC records
int epcCount = 0;             // Counter for unique EPCs


const struct device *myuart1 = DEVICE_DT_GET(DT_NODELABEL(uart1));

struct uart_config uart_cfg = {
    .baudrate = 9600,
    .data_bits = UART_CFG_DATA_BITS_8,
    .parity = UART_CFG_PARITY_NONE,
    .stop_bits = UART_CFG_STOP_BITS_1,
    .flow_ctrl = UART_CFG_FLOW_CTRL_NONE,

};

static void uart_cb(const struct device *dev, struct uart_event *evt, void *user_data)
{

    switch (evt->type) {
    case UART_TX_DONE:
        // Transmission completed
        printk("UART TX completed\n");
        break;

    case UART_TX_ABORTED:
        // trasmission aborted
        printk("UART Tx aborted");
        break;

    case UART_RX_RDY:
        // Data is ready in the buffer
        printk("Received data: ");
        for (int i = evt->data.rx.offset; i < evt->data.rx.offset + evt->data.rx.len; i++) {
            printk("%c", rx_buf[i]);
        }
        printk("\n");
        data_received = true;
        break;

    case UART_RX_BUF_REQUEST:
        // Provide the next buffer
        uart_rx_buf_rsp(dev, rx_buf, sizeof(rx_buf));
        break;

    case UART_RX_BUF_RELEASED:
        // A buffer was released
        break;

    case UART_RX_DISABLED:
        printk("UART RX disabled\n");
        break;

    case UART_RX_STOPPED:
        printk("UART RX stopped\n");
        break;



	default:
		break;
	}
}

/*
static void uart_callback(const struct device *dev, struct uart_event *evt, void *user_data) {
    switch (evt->type) {
    case UART_TX_DONE:
        // Transmission completed
        printk("UART TX completed\n");
        break;

    case UART_TX_ABORTED:
        // trasmission aborted
        printk("UART Tx aborted");
        break;

    case UART_RX_RDY:
        // Data is ready in the buffer
        printk("Received data: ");
        for (int i = evt->data.rx.offset; i < evt->data.rx.offset + evt->data.rx.len; i++) {
            printk("%c", rx_buf[i]);
        }
        printk("\n");
        data_received = true;
        break;

    case UART_RX_BUF_REQUEST:
        // Provide the next buffer
        uart_rx_buf_rsp(dev, rx_buf, sizeof(rx_buf));
        break;

    case UART_RX_BUF_RELEASED:
        // A buffer was released
        break;

    case UART_RX_DISABLED:
        printk("UART RX disabled\n");
        break;

    case UART_RX_STOPPED:
        printk("UART RX stopped\n");
        break;

    default:
        break;
    }
}

*/




void send_str(const struct device *uart,  char *str, size_t size)
{
int ret;

	ret = uart_tx(uart, str, size, SYS_FOREVER_US);
	if (ret) {
		printf("Got 1");
	}

    printf("\n");
k_sleep(K_MSEC(10));
}

void recv_str(const struct device *uart, char *str)
{
int ret;

	ret = uart_rx_enable(uart ,str,6,100);
	if (ret) {
		printf("Got 1");
	}
    printf("\n");
}

int main(void)
{
int rc;
int ret;

// unsigned char recv_buf[]={"here i am "};
int i = 10;


if (!device_is_ready(myuart1)) 
{
    printf("UART1 device not ready\n");
} else {
    printf("UART1 device ready\n");

}

rc = uart_configure(myuart1, &uart_cfg);
if (rc != 0) {
        printf("Could not configure device uart1\n");
} else {
        printf("Configure device uart1\n");
}


ret = uart_callback_set(myuart1, uart_cb, NULL);
if (ret) {
    printk("Failed to set UART callback: %d\n", ret);
    return 1;
}

/*ret = uart_rx_enable(myuart1, rx_buf, sizeof(rx_buf), UART_RX_TIMEOUT);
if (ret) {
        printk("Failed to enable UART RX: %d\n", ret);
        return 1;
}*/


    char numberStr[2];        // Buffer for numeral as a string




    // Initial command to send
//    unsigned char init_command[] = {0xC8, 0x8C, 0x00, 0x0A, 0x82, 0x00, 0x00, 0x88, 0x0D, 0x0A};
//    send_str(myuart1, init_command, sizeof(init_command));
unsigned char init_msg[] = {"hello from nordic - "};
unsigned char init_command[sizeof(init_msg)+2];
//    send_str(myuart1, init_command, sizeof(init_command));


int j;
j = 10;

while (j--){

    ret = uart_rx_enable(myuart1, rx_buf, sizeof(rx_buf), UART_RX_TIMEOUT);
    if (ret) {
        printk("Failed to enable UART RX: %d\n", ret);
        return 1;
    }
    snprintf(numberStr, sizeof(numberStr), "%d", j);


    strcpy(init_command, init_msg);
    strcat(init_command, numberStr);

    send_str(myuart1, init_command, sizeof(init_command));

    i = 10;
    while (i--) {

        if (data_received) {
            data_received = false;
            // Additional processing can go here
		    printf("Got 1");
        }
        k_sleep(K_MSEC(100));
    }
    printf("the end");
//         send_str(uart1, send_buf);   return 0;
};
}